END_VIEW_NAME = "View"
INDEX_VIEW_PREFIX = "Index"

def _get_module(view):

    return view.__module__.split(".")[-1]

def _get_name(view):

    if END_VIEW_NAME in view.__name__:
        return view.__name__[:-len(END_VIEW_NAME)].lower()
    return view.__name__.lower()

def get_template_name(view):

    module = _get_module(view)
    name = _get_name(view)
    return "%s/%s.html" % (module, name)

def get_url(view):

    module = _get_module(view)
    name = _get_name(view)

    if name == INDEX_VIEW_PREFIX.lower():
        return r"^%s/$" % module

    return r"^%s/%s/$" % (module, name)

def get_resource(view):

    return _get_name(view)


from django.views.generic import View


def is_sublclass(klass, super_class):

    try:
        return issubclass(klass, super_class) and not klass is super_class
    except:
        return False


def is_valid_view(value, root_module):

    return is_sublclass(value, View) and hasattr(value, "__module__") and root_module in value.__module__



from introspector import Introspector


class UrlsManager(object):

    instrospector = Introspector()

    def __init__(self, urlpatterns, views_root):

        self.urlpatterns = urlpatterns
        self.views_root = views_root
        self.add_views()

    def add_views(self):

        views = self.instrospector.infer_views(self.views_root)

        for view in views:

            django_urls = self.instrospector.get_django_urls(view)

            for url in django_urls:
                self.urlpatterns.append(url)



from django.views.generic import View
from .. import conventions


class RESTView(View):

    @classmethod
    def _infer_rest_urls(self):

        if "resource" in self.__dict__:
            resource = self.resource
        else:
            resource = conventions.get_resource(self)

        self.url = [r"^%s/(?P<id>\d)/$" % resource, r"^%s/$" % resource]
        return self


import pkgutil

from django.conf.urls.defaults import url as djangourl
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_view_exempt
from utils import is_valid_view

from rest import RESTView

import conventions


class Introspector(object):

    def infer_views(self, views_root):

        views = []

        for loader, module_name, is_pkg in pkgutil.walk_packages(views_root.__path__, "%s." % views_root.__name__):
            if not is_pkg:
                views.extend(self._get_module_views(loader, module_name, views_root))

        return views

    def _get_module_views(self, loader, module_name, views_root):

        module = loader.find_module(module_name).load_module(module_name)
        return [value for value in module.__dict__.values() if is_valid_view(value, views_root.__name__)]

    def get_django_urls(self, view):
        
        return self._get_urls(view)

    def _infer_methods(self, view):

        if issubclass(view, RESTView):
            return view._infer_rest_urls()

        self._infer_template(view)
        self._infer_url(view)
        return view

    def _infer_template(self, view):

        if "template_name" not in view.__dict__:
            view.template_name = conventions.get_template_name(view)

    def _infer_url(self, view):

        if "url" not in view.__dict__:
            view.url = conventions.get_url(view)

    def _check_for_boolean(self, view, django_view, name, apply_function, negative=False):

        boolean = getattr(view, name, False)
        if negative and not boolean or not negative and boolean:
            django_view = apply_function(django_view)
        return django_view

    def _get_urls(self, view):

        view = self._infer_methods(view)

        django_view = view.as_view()
        django_view = self._check_for_boolean(view, django_view, "login_exempt", login_required)
        django_view = self._check_for_boolean(view, django_view, "csrf_exempt", csrf_view_exempt)

        if isinstance(view.url, list):
            return [self._get_django_url(django_view, view, url) for url in view.url]
        
        return [self._get_django_url(django_view, view, view.url)]

    def _get_django_url(self, django_view, view, url):

        params = self._check_for_params(view)
        return djangourl(url, django_view, params)

    def _check_for_params(self, view):

        return getattr(view, 'url_params', {})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="django_conventions",
    version="0.1",
    description="Django Convention Over Configuration",
    author="Juan Manuel García",
    author_email = "jmg.utn@gmail.com",
    license = "GPL v3",
    keywords = "Django Routing Convetion Over Configuration",
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "django",
    ],
    url='https://github.com/jmg/django_conventions',
)
